# -*- coding: utf-8 -*-

import bpy
import spb
from   Spr               import *
from   spb.utils         import *


class Handler():
	'''ハンドラ：Sprとbpyをつなぐユニットのベースクラス'''

	def __init__(self):
		# シンクロナイザ
		self.synchronizers = []

		# Springheadオブジェクトの構築が正しく完了しているか
		self.build_finished = False

		# Springheadオブジェクトの構築を行ったかどうか．依存関係にあるハンドラがspr再構築を行うかの目印
		self.updated = False

		# 依存関係にあるハンドラ．相手がspr再構築した（=updated）場合こちらも自動で再構築する．
		self.dependent_handlers = []

		# Springheadオブジェクトの構築に必要な情報．
		self.build_info = {}

	# Obsolete<!!>
	def get_synchronizer(self, synctype):
		'''synctypeで指定されるクラスのシンクロナイザを取得する'''
		for synchronizer in self.synchronizers:
			if type(synchronizer) is synctype:
				return synchronizer

	def before_sync(self):
		'''Synchronize前に行う処理を記述する．オーバーライド用．'''
		pass

	def after_sync(self):
		'''Synchronize後に行う処理を記述する．オーバーライド用．'''
		pass

	def sync(self):
		'''bpy<=>spr間の同期を行う．'''
		# 同期
		if self.build_finished: # 構築が正しく完了するまでは動かない
			for synchronizer in self.synchronizers:
				if self.updated or synchronizer.bpy_sync_cond():
					new_bpy = synchronizer.get_bpy_value_mod(synchronizer.bpy)
					synchronizer.bpy = copy(new_bpy)
					synchronizer.spr = to_spr(new_bpy)
					synchronizer.set_spr_value(synchronizer.spr)

				elif synchronizer.is_spr_changed():
					new_spr = synchronizer.get_spr_value_mod(synchronizer.spr)
					synchronizer.bpy = to_bpy(new_spr)
					synchronizer.spr = copy(new_spr)
					synchronizer.set_bpy_value(synchronizer.bpy)

	def sync_spr_to_bpy(self):
		'''spr=>bpyの反映のみ行う（更新チェックもしない）．'''
		for synchronizer in self.synchronizers:
			new_spr = synchronizer.get_spr_value_mod(synchronizer.spr)
			synchronizer.bpy = to_bpy(new_spr)
			synchronizer.spr = copy(new_spr)
			synchronizer.set_bpy_value(synchronizer.bpy)

	def store(self, key, init=False):
		'''状態の一時保存を行う．'''
		for synchronizer in self.synchronizers:
			if synchronizer.storable():
				synchronizer.store(key, init)

	def restore(self, key):
		'''保存しておいた状態に戻す．'''
		for synchronizer in self.synchronizers:
			if synchronizer.storable():
				synchronizer.restore(key)

	# --

	def update_build_info(self):
		'''Springheadオブジェクトの構築に必要な情報を更新．これが変化すると再構築される'''
		pass

	def build(self):
		'''対応するSpringhead側オブジェクトを構築する．依存ハンドラの取得も行う．'''
		pass

	def post_build(self):
		'''Springheadオブジェクト構築後に行う必要のある後処理'''
		pass

	def destroy(self):
		'''Springheadオブジェクトを削除する．存在しないものものを消さないように気をつけること'''
		pass

	# --

	def get_object(self):
		'''このハンドラが対応する（広義の）bpyオブジェクト．削除判定に使う'''
		return self.object

	def must_removed(self):
		'''このハンドラはもはや消滅すべき状態なのではないだろうか'''
		o = self.get_object()
		return (not o.alive) or (not type(self).is_target(o))

	def step(self):
		'''1ステップごとに実行する内容を記述する．Ruleの実行など'''
		pass

	def record(self, frame):
		'''キーフレームへの記録を行う．'''
		pass

	def __str__(self):
		return( type(self).__name__ + "<" + self.name + ">" )
