# -*- coding: utf-8 -*-

import bpy
import spb

from   mathutils        import Vector,Quaternion
from   spb.handler      import Handler
from   spb.spbapi       import spbapi

from   Spr              import *
from   spb.synchronizer import *
from   spb.utils        import *
from   spb.scriptengine import *


# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
#
# Handler
#

class CreatureHandler(Handler):
	'''Spr.CRCreatureとbpy.groupを同期するクラス．'''

	def __init__(self, group, crCreature=None):
		Handler.__init__(self)

		self.group      = group
		self.name       = group.name

		self.crCreature = crCreature

		self.rule       = ScriptEngine()
		self.rule.set_name("creature_rule_"+self.group.name)
		self.first_step = True

		# -- Synchronizers
		# (None)

	# --- --- --- --- ---
	# 基本クラスのオーバーライド

	def get_object(self):
		'''このハンドラが対応する（広義の）bpyオブジェクト．削除判定に使う'''
		# Creatureの場合，objectではなくgroupに対して作られるので．
		return self.group

	def build(self):
		'''sprのオブジェクトを構築する'''
		# 旧オブジェクトがあれば消しておく
		self.destroy()

		# Create Creature
		self.crCreature = spbapi.GetCRSdk().CreateCreature(CRCreature().GetIfInfoStatic(), CRCreatureDesc())
		self.crCreature.AddChildObject(spbapi.GetFWSdk().GetScene(0).GetPHScene())

		# Create Body
		self.crCreature.CreateBody(CRBody().GetIfInfoStatic(), CRBodyDesc())

		# Set Name
		self.crCreature.SetName("creature_"+self.group.name)
		self.crCreature.GetBody(0).SetName("body_"+self.group.name)

		# 構築完了フラグを更新
		self.build_finished = ( (not self.crCreature is None) and (self.crCreature.NBodies() > 0) )

	def step(self):
		'''1ステップごとに実行する内容を記述する．Ruleの実行など'''
		if bpy.context.scene.spr_creature_enabled==1 and bpy.context.scene.spr_step_enabled==1:
			if self.first_step:
				self.rule.update()
				self.first_step = False
			self.rule.step()

	def destroy(self):
		if not self.crCreature is None:
			# <!!> To Be Implemented
			self.crCreature = None
		self.build_finished = False

	@classmethod
	def is_target(cls, group):
		'''オブジェクトがこのハンドラの対象となるべきものであるかを返す'''
		is_creature = (group.spr_creature_group==1)
		return is_creature


