# -*- coding: utf-8 -*-

import bpy
import sys
import traceback


class ScriptEngine():
	def __init__(self):
		self.initRule   = ""
		self.stepRule   = ""

	def execute(self, script):
		'''scriptを実行する．'''
		try:
			exec(script, self.globals)
		except Exception as e:
			print(traceback.format_exc())

	def reset(self):
		'''変数類をリセットする．'''
		self.globals = {}

	def set_name(self, name):
		'''スクリプト名をセットする．'''
		self.name = name

	def update(self):
		'''テキストエディタからスクリプトを再読み込みする．'''
		self.reset()
		self.initRule = self.update_rule(self.name+"_init")
		self.stepRule = self.update_rule(self.name+"_step")
		self.init()

	def init(self):
		'''初期化スクリプトを実行する．'''
		self.execute(self.initRule)

	def step(self):
		'''ステップスクリプトを実行する．'''
		self.execute(self.stepRule)

	def update_rule(self, rule_name):
		# 無ければ作る
		if not rule_name in bpy.data.texts:
			bpy.data.texts.new(rule_name)
		textedit = bpy.data.texts[rule_name]

		# .blendと同じディレクトリのファイルをimportできるようパスをセットしておく
		bnlen = len(bpy.path.basename(bpy.data.filepath))
		dir = bpy.data.filepath[0:-bnlen]
		if not dir in sys.path:
			sys.path.append(dir)

		# ファイル(External)ならリロードする
		if not textedit.filepath=='':
			# スクリプト内で同じディレクトリのファイルをimportできるようパスをセットしておく
			bnlen = len(bpy.path.basename(textedit.filepath))
			dir = textedit.filepath[0:-bnlen]
			if not dir in sys.path:
				sys.path.append(dir)

			# スクリプトのリロード
			try:
				with open(textedit.filepath) as ifp:
					textedit.from_string(ifp.read())
			except FileNotFoundError:
				print("Error : Rule Text Not Found for ", rule_name)

		return textedit.as_string()

