# -*- coding: utf-8 -*-

import bpy
import spb

from   Spr              import *
from   spb.synchronizer import *
from   spb.utils        import *


# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
#
# SprObjectパネルのレイアウト
#

class SprPhysicsPanel(bpy.types.Panel):
	bl_label	   = "Springhead Physics"
	bl_space_type  = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context	   = "object"

	def draw(self, context):
		layout = self.layout
		obj	   = context.object
		mat	   = obj.material_slots.id_data.active_material

		# -- 
		split = layout.split()
		row = split.row()
		row.prop(obj, "spr_object_type", text="")

		if obj.spr_object_type=="Solid":
			row = split.row(align=True)
			row.prop(obj, "spr_solid_props_hold", text="Hold", icon=("PINNED" if obj.spr_solid_props_hold else "UNPINNED"))
			if obj.spr_solid_props_hold==1:
				row.prop(obj, "spr_solid_props_apply", text="Apply", toggle=True)

			self.draw_solid(context)

		elif obj.spr_object_type=="Joint":
			row = split.row(align=True)
			row.prop(obj, "spr_joint_props_hold", text="Hold", icon=("PINNED" if obj.spr_joint_props_hold else "UNPINNED"))
			if obj.spr_joint_props_hold==1:
				row.prop(obj, "spr_joint_props_apply", text="Apply", toggle=True)

			self.draw_joint(context)
		

	def draw_solid(self, context):
		layout = self.layout
		obj	   = context.object
		mat	   = obj.material_slots.id_data.active_material

		# -- 
		split = layout.split()
		row1 = split.row()
		row1.prop(obj, "spr_shape_type",   text="Shape")
		row2 = split.row()
		row2.operator("spr.shape_renew",   text="Renew")
		row2.prop(obj, "spr_solid_static", text="Static Solid")
		
		# -- 
		split = layout.split()
		col = split.column()
		col.prop(obj.game, "mass",             text="Mass")

		col2 = col.column(align=True)
		col2.label(text="Center of Mass")
		col2.prop(obj, "spr_center_x", text="x")
		col2.prop(obj, "spr_center_y", text="y")
		col2.prop(obj, "spr_center_z", text="z")
		col2.operator("spr.solid_center_from_cursor", "From Cursor")
				
		# -- 
		col = split.column()
		col.label(text="Material:")
		if (not mat==None):
			col.prop(mat.physics, "elasticity",          text="Elasticity")
			col.prop(mat.physics, "friction",            text="Friction")
			col.prop(mat,         "spr_friction_static", text="Static Friction")
		else:
			col.label(text="<Please Add Material>")

		# -- 
		split = layout.split()
		col = split.column(align=True)
		col.prop(obj, "spr_autoset_inertia", text="Automatic Inertia Setting")
		if obj.spr_autoset_inertia==0:
			row = col.row()
			row.prop(obj, "spr_inertia_x",  text="X")
			row.prop(obj, "spr_inertia_xy", text="xy")
			row.prop(obj, "spr_inertia_xz", text="xz")
			row = col.row()
			row.prop(obj, "spr_inertia_yx", text="yx")
			row.prop(obj, "spr_inertia_y",  text="Y")
			row.prop(obj, "spr_inertia_yz", text="yz")
			row = col.row()
			row.prop(obj, "spr_inertia_zx", text="zx")
			row.prop(obj, "spr_inertia_zy", text="zy")
			row.prop(obj, "spr_inertia_z",  text="Z")
			row = col.row()
			row.operator("spr.solid_inertia_calc", "Auto")
			row.operator("spr.solid_inertia_zero", "Zero")
			row.operator("spr.solid_inertia_unit", "Unit")

		# --
		split = layout.split()
		row = split.row()
		row.prop(obj, "spr_connect_interface",   text="Human Interface")

		# --- --- --- --- ---
		# Solid Control Panel
		box   = layout.box()
		split = box.split()
		col = split.column()
		col.prop(obj, "spr_show_solid_control", text="Show Solid Control Panel")

		if obj.spr_show_solid_control==1:
			box2 = box.box()
			split = box2.split()
			col1 = split.column()
			col1.label(text="Velocity")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_solid_velocity_hold", text="Hold", icon=("PINNED" if obj.spr_solid_velocity_hold else "UNPINNED"))
			if obj.spr_solid_velocity_hold==1:
				row.prop(obj, "spr_solid_velocity_apply", text="Apply", toggle=True)
			split = box2.split()
			row = split.row(align=True)
			row.prop(obj, "spr_solid_velocity_local", text="Local")
			row.prop(obj, "spr_solid_velocity_x", text="x")
			row.prop(obj, "spr_solid_velocity_y", text="y")
			row.prop(obj, "spr_solid_velocity_z", text="z")

			# --
			box2 = box.box()
			split = box2.split()
			col1 = split.column()
			col1.label(text="Angular Velocity")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_solid_angvelocity_hold", text="Hold", icon=("PINNED" if obj.spr_solid_angvelocity_hold else "UNPINNED"))
			if obj.spr_solid_angvelocity_hold==1:
				row.prop(obj, "spr_solid_angvelocity_apply", text="Apply", toggle=True)
			split = box2.split()
			row = split.row(align=True)
			row.prop(obj, "spr_solid_ang_velocity_local", text="Local")
			row.prop(obj, "spr_solid_angular_velocity_x", text="x")
			row.prop(obj, "spr_solid_angular_velocity_y", text="y")
			row.prop(obj, "spr_solid_angular_velocity_z", text="z")

			# --
			box2 = box.box()
			split = box2.split()
			col1 = split.column()
			col1.label(text="Force")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_solid_force_hold", text="Hold", icon=("PINNED" if obj.spr_solid_force_hold else "UNPINNED"))
			if obj.spr_solid_force_hold==1:
				row.prop(obj, "spr_solid_force_apply", text="Apply", toggle=True)
			split = box2.split()
			row = split.row(align=True)
			row.prop(obj, "spr_solid_force_local", text="Local")
			row.prop(obj, "spr_solid_force_x", text="x")
			row.prop(obj, "spr_solid_force_y", text="y")
			row.prop(obj, "spr_solid_force_z", text="z")
			split = box2.split()
			row = split.row(align=True)
			row.prop(obj, "spr_solid_set_force_point", text="Set Apply Point")
			if obj.spr_solid_set_force_point:
				row.prop(obj, "spr_solid_force_point_x", text="x")
				row.prop(obj, "spr_solid_force_point_y", text="y")
				row.prop(obj, "spr_solid_force_point_z", text="z")

			# --
			box2 = box.box()
			split = box2.split()
			col1 = split.column()
			col1.label(text="Torque")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_solid_torque_hold", text="Hold", icon=("PINNED" if obj.spr_solid_torque_hold else "UNPINNED"))
			if obj.spr_solid_torque_hold==1:
				row.prop(obj, "spr_solid_torque_apply", text="Apply", toggle=True)
			split = box2.split()
			row = split.row(align=True)
			row.prop(obj, "spr_solid_torque_local", text="Local")
			row.prop(obj, "spr_solid_torque_x", text="x")
			row.prop(obj, "spr_solid_torque_y", text="y")
			row.prop(obj, "spr_solid_torque_z", text="z")


		# --- --- --- --- ---
		# IK Endeffector
		box   = layout.box()
		split = box.split()
		col = split.column()
		col.prop(obj, "spr_ik_enabled", text="IK Endeffector")

		if obj.spr_ik_enabled==1:
			split = box.split()
			col = split.column()
			col.label("Target Local Position")
			row = col.row(align=True)
			row.prop(obj, "spr_ik_target_local_pos_x", text="x")
			row.prop(obj, "spr_ik_target_local_pos_y", text="y")
			row.prop(obj, "spr_ik_target_local_pos_z", text="z")
			row.operator("spr.ik_target_local_from_cursor",text = "From Cursor")

			split = box.split()
			row = split.row()
			row.prop(obj, "spr_ik_pos_control_enabled", text="Position Control")
			row.prop(obj, "spr_ik_ori_control_enabled", text="Orientation Control")
			
			# --- --- --- --- ---
			# IK Control
			box   = layout.box()
			split = box.split()
			row = split.row()
			row.prop(obj, "spr_show_ik_control", text="Show IK Control Panel")

			if obj.spr_show_ik_control==1:
				row2 = row.row(align=True)
				row2.prop(obj, "spr_ik_targetpose_hold", text="Hold", icon=("PINNED" if obj.spr_ik_targetpose_hold else "UNPINNED"))
				if obj.spr_ik_targetpose_hold==1:
					row2.prop(obj, "spr_ik_targetpose_apply", text="Apply", toggle=True)

				split = box.split()
				row = split.row()
				row.prop(obj, "spr_ik_target_object_enabled", "Use Target Object")
				row2 = row.row()
				row2.active = (obj.spr_ik_target_object_enabled==1)
				row2.prop(obj, "spr_ik_target_object_name", text="")

				if obj.spr_ik_target_object_enabled==0:
					split = box.split()
					col  = split.column()
					col2 = col.column(align=True)
					col2.label(text="Target Position:")
					col2.prop(obj, "spr_ik_target_pos_x", text="x")
					col2.prop(obj, "spr_ik_target_pos_y", text="y")
					col2.prop(obj, "spr_ik_target_pos_z", text="z")

					col = split.column()
					col2 = col.column(align=True)
					col2.label(text="Target Orientation (Deg):")
					col2.prop(obj, "spr_ik_target_ori_x", text="x")
					col2.prop(obj, "spr_ik_target_ori_y", text="y")
					col2.prop(obj, "spr_ik_target_ori_z", text="z")


	def draw_joint(self, context):
		layout = self.layout
		
		obj = context.object
		scene = context.scene
		
		# -- 
		split = layout.split()
		row = split.row()
		row.prop(obj, "spr_joint_mode",    text="Type")
		row.prop(obj, "spr_joint_enabled", text="Enable Joint")
		
		# -- 
		box = layout.box()
		split = box.split()
		row = split.row(align=True)
		row.prop(obj, "spr_joint_socket_target",       text="Socket")
		row2 = row.row()
		row2.active = (obj.spr_joint_socket_target=="Solid Name" or obj.spr_joint_socket_target=="Axis Name")
		row2.prop(obj, "spr_joint_socket_target_name", text="Target")

		split = box.split()
		row = split.row(align=True)
		row.prop(obj, "spr_joint_plug_target",         text="Plug")
		row2 = row.row()
		row2.active=(obj.spr_joint_plug_target=="Solid Name")
		row2.prop(obj, "spr_joint_plug_target_name",   text="Target")

		# -- 
		split = layout.split()
		col = split.column()
		if (obj.spr_joint_mode=="Spring"):
			col2 = col.column(align=True)
			col2.label(text ="Spring:")
			col2.prop(obj, "spr_spring_x",   text="x")
			col2.prop(obj, "spr_spring_y",   text="y")
			col2.prop(obj, "spr_spring_z",   text="z")
			col2.prop(obj, "spr_spring_ori", text="ori")

			col2 = col.column(align=True)
			col2.label(text ="Damper:")
			col2.prop(obj, "spr_damper_x",   text="x")
			col2.prop(obj, "spr_damper_y",   text="y")
			col2.prop(obj, "spr_damper_z",   text="z")
			col2.prop(obj, "spr_damper_ori", text="ori")
		else:
			col.prop(obj,  "spr_spring",     text="Spring")
			col.prop(obj,  "spr_damper",     text="Damper")

		col.prop(obj, "spr_joint_max_force", text="Max Force")
				
		# -- 
		col = split.column()
		col.prop(obj,      "spr_joint_collision", text="Collision")
		row = col.row()
		row.active = (obj.spr_joint_mode=="Hinge")
		row.prop(obj,      "spr_joint_cyclic",    text="Cyclic")

		# --- --- --- --- ---
		# Joint Limit
		box   = layout.box()
		split = box.split()
		col = split.column()
		col.prop(obj, "spr_joint_limit_enable", text="Limit")

		if obj.spr_joint_limit_enable==1:
			if obj.spr_joint_mode=="Hinge" or obj.spr_joint_mode=="Slider":
				split = box.split()
				row = split.row(align=True)
				row.prop(obj, "spr_limit_range_min",   text="min")
				row.operator("spr.limit_inf_range_min",text="inf")
				row.prop(obj, "spr_limit_range_max",   text="Max")
				row.operator("spr.limit_inf_range_max",text="inf")

			elif obj.spr_joint_mode=="Ball":
				split = box.split()
				row = split.row(align=True)
				row.label(text="Swing")
				row.prop(obj, "spr_limit_swing_min",   text="min")
				row.operator("spr.limit_inf_swing_min",text="inf")
				row.prop(obj, "spr_limit_swing_max",   text="Max")
				row.operator("spr.limit_inf_swing_max",text="inf")
				
				split = box.split()
				row = split.row(align=True)
				row.label(text="Twist")
				row.prop(obj, "spr_limit_twist_min",   text="min")
				row.operator("spr.limit_inf_twist_min",text="inf")
				row.prop(obj, "spr_limit_twist_max",   text="Max")
				row.operator("spr.limit_inf_twist_max",text="inf")

			split = box.split()
			row = split.row(align=True)
			row.prop(obj, "spr_limit_spring", text="Spring")
			row.prop(obj, "spr_limit_damper", text="Damper")

		# --- --- --- --- ---
		# Joint Control Panel
		box   = layout.box()
		split = box.split()
		col = split.column()
		col.prop(obj, "spr_show_joint_control", text="Show Joint Control Panel")

		if obj.spr_show_joint_control==1:
			split = box.split()
			col1 = split.column()
			if obj.spr_joint_mode=="Ball":
				col1.label(text="Target Position")
			else:
				col1.prop(obj, "spr_joint_target_position", text="Target Position")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_joint_targetposition_hold", text="Hold", icon=("PINNED" if obj.spr_joint_targetposition_hold else "UNPINNED"))
			if obj.spr_joint_targetposition_hold==1:
				row.prop(obj, "spr_joint_targetposition_apply", text="Apply", toggle=True)
			if obj.spr_joint_mode=="Ball":
				split = box.split()
				row = split.row(align=True)
				row.prop(obj, "spr_joint_target_position_x", text="x")
				row.prop(obj, "spr_joint_target_position_y", text="y")
				row.prop(obj, "spr_joint_target_position_z", text="z")

			# -- 
			split = box.split()
			col1 = split.column()
			if obj.spr_joint_mode=="Ball":
				col1.label(text="Target Velocity")
			else:
				col1.prop(obj, "spr_joint_target_velocity", text="Target Velocity")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_joint_velocity_hold", text="Hold", icon=("PINNED" if obj.spr_joint_velocity_hold else "UNPINNED"))
			if obj.spr_joint_velocity_hold==1:
				row.prop(obj, "spr_joint_velocity_apply", text="Apply", toggle=True)
			if obj.spr_joint_mode=="Ball":
				split = box.split()
				row = split.row(align=True)
				row.prop(obj, "spr_joint_target_velocity_x", text="x")
				row.prop(obj, "spr_joint_target_velocity_y", text="y")
				row.prop(obj, "spr_joint_target_velocity_z", text="z")

			# -- 
			split = box.split()
			col1 = split.column()
			if obj.spr_joint_mode=="Ball":
				col1.label(text="Offset Force")
			else:
				col1.prop(obj, "spr_joint_offset_force", text="Offset Force")
			col2 = split.column()
			row = col2.row(align=True)
			row.prop(obj, "spr_joint_offsetforce_hold", text="Hold", icon=("PINNED" if obj.spr_joint_offsetforce_hold else "UNPINNED"))
			if obj.spr_joint_offsetforce_hold==1:
				row.prop(obj, "spr_joint_offsetforce_apply", text="Apply", toggle=True)
			if obj.spr_joint_mode=="Ball":
				split = box.split()
				row = split.row(align=True)
				row.prop(obj, "spr_joint_offset_force_x", text="x")
				row.prop(obj, "spr_joint_offset_force_y", text="y")
				row.prop(obj, "spr_joint_offset_force_z", text="z")


		# --- --- --- --- ---
		# IK Actuator
		box   = layout.box()
		split = box.split()
		col = split.column()
		col.prop(obj, "spr_ik_enabled", text="IK Actuator")

		if obj.spr_ik_enabled==1:
			split = box.split()
			row = split.column()
			row.prop(obj, "spr_ik_bias", "Bias")


# --- --- --- --- --- --- --- --- --- --- 
# ボタンを押した際の動作

# 剛体のShapeのRenewボタンを押した際の動作
class OBJECT_OT_SprShapeRenew(bpy.types.Operator):
	bl_idname      = "spr.shape_renew"
	bl_label       = "[spr]Shape Renew"
	bl_description = "Apply update on bpy shape into spr shape"
	
	def execute(self,context):
		obj = context.object
		
		handler = spb.handlers.shape_handlers[obj.name]
		if not handler is None:
			handler.build()
		
		return{'FINISHED'}


# 剛体CenterのFrom Cursorボタンを押した際の動作
class OBJECT_OT_SprSolidCenterFromCursor(bpy.types.Operator):
	bl_idname      = "spr.solid_center_from_cursor"
	bl_label       = "[spr]Set Center to Cursor Position"
	bl_description = "[Spr]"
	
	def execute(self,context):
		obj = context.object
		
		handler = spb.handlers.solid_handlers[obj.name]
		if not handler is None:
			cursor_pos = bpy.context.scene.cursor_location
			cursor_pos = Vec3d(cursor_pos.x, cursor_pos.y, cursor_pos.z)
			local_pos  = handler.phSolid.GetPose().Inv().transform(cursor_pos)
			so_con.object.spr_center_x = local_pos.x
			so_con.object.spr_center_y = local_pos.y
			so_con.object.spr_center_z = local_pos.z
		
		return{'FINISHED'}


# 剛体のInertia Autoボタンを押した際の動作
class OBJECT_OT_SprSolidInertiaCalc(bpy.types.Operator):
	bl_idname      = "spr.solid_inertia_calc"
	bl_label       = "[spr]Calc Inertia Tensor"
	bl_description = "[Spr]"
	
	def execute(self,context):
		obj = context.object
		
		handler = spb.handlers.solid_handlers[obj.name]
		if not handler is None:
			pass
		
		return{'FINISHED'}


# 剛体のInertia Zeroボタンを押した際の動作
class OBJECT_OT_SprSolidInertiaZero(bpy.types.Operator):
	bl_idname      = "spr.solid_inertia_zero"
	bl_label       = "[spr]Set Inertia Tensor to Zero"
	bl_description = "[Spr]"
	
	def execute(self,context):
		obj = context.object
		obj.spr_inertia_x  = 0
		obj.spr_inertia_xy = 0
		obj.spr_inertia_xz = 0
		obj.spr_inertia_yx = 0
		obj.spr_inertia_y  = 0
		obj.spr_inertia_yz = 0
		obj.spr_inertia_zx = 0
		obj.spr_inertia_zy = 0
		obj.spr_inertia_z  = 0
		
		return{'FINISHED'}


# 剛体のInertia Unitボタンを押した際の動作
class OBJECT_OT_SprSolidInertiaUnit(bpy.types.Operator):
	bl_idname      = "spr.solid_inertia_unit"
	bl_label       = "[spr]Set Inertia Tensor to Unit Matrix"
	bl_description = "[Spr]"
	
	def execute(self,context):
		obj = context.object
		obj.spr_inertia_x  = 1
		obj.spr_inertia_xy = 0
		obj.spr_inertia_xz = 0
		obj.spr_inertia_yx = 0
		obj.spr_inertia_y  = 1
		obj.spr_inertia_yz = 0
		obj.spr_inertia_zx = 0
		obj.spr_inertia_zy = 0
		obj.spr_inertia_z  = 1
		
		return{'FINISHED'}


# IK EndeffectorのFrom Cursorボタンを押した時の動作
class OBJECT_OT_SprIKTargetLocalFromCursor(bpy.types.Operator):
	bl_idname = "spr.ik_target_local_from_cursor"
	bl_label = "[spr]Set IK Target Local Position from Cursor"
	bl_description = "[Spr]"
	
	def execute(self,context):
		obj = context.object

		ikeff_hnd = spb.handlers.ikeff_handlers[obj.name]
		if ikeff_hnd:
			cursor_pos = to_spr(bpy.context.scene.cursor_location)
			local_pos  = ikeff_hnd.solidHnd.phSolid.GetPose().Inv().transform(cursor_pos)
			ikeff_hnd.object.spr_ik_target_local_pos_x = local_pos.x
			ikeff_hnd.object.spr_ik_target_local_pos_y = local_pos.y
			ikeff_hnd.object.spr_ik_target_local_pos_z = local_pos.z
		
		return{'FINISHED'}



# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
#
# SprMaterialパネルのレイアウト
#

#Spr Object Panelでも設定できるがMaterialパネルにも表示する
class SprMaterialPanel(bpy.types.Panel):
	bl_label = "Springhead Material"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "material"

	def draw(self, context):
		layout = self.layout

		obj = context.object
		mat = obj.material_slots.id_data.active_material

		split = layout.split()
		row = split.row(align=True)
		row.prop(obj, "spr_solid_props_hold", text="Hold", icon=("PINNED" if obj.spr_solid_props_hold else "UNPINNED"))
		if obj.spr_solid_props_hold==1:
			row.prop(obj, "spr_solid_props_apply", text="Apply", toggle=True)

		split = layout.split()
		col = split.column()
		if(mat!=None):
			col.prop(mat.physics, "elasticity",  text="Elasticity")
			col.prop(mat.physics, "friction",    text="Friction")
			col.prop(mat, "spr_friction_static", text="Static Friction")



# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
#
# Creatureパネルのレイアウト
#

class SprCreaturePanel(bpy.types.Panel):
	bl_label = "Springhead Creature"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "object"

	def draw(self, context):
		layout = self.layout

		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		if((not group is None) and (obj.name in spb.handlers.solid_handlers)):
			layout.label(text = "Name: " + group.name)
			
			layout.prop(obj,"spr_crbone_label", text="Bone Label")

			row = layout.row()
			row.prop(group,"spr_creature_spring_ratio", text="Spring Ratio")
			row.prop(group,"spr_creature_damper_ratio", text="Damper Ratio")
			
			split = layout.split()
			col = split.column(align=True)
			col.label(text="Parameters: ")
			row = col.row(align=True)
			for param in group.spr_creature_parameters:
				row = col.row(align=True)
				row.context_pointer_set("object", param)
				row.prop(param, "name",  text="")
				row.prop(param, "value", text="")
				row.operator("spr.cr_del_parameter", text="", icon='X')
			col.operator("spr.cr_add_parameter", text="Add Parameter", icon='ZOOMIN')
				
			split = layout.split()
			col = split.column(align=True)
			col.label(text="Visualizers: ")
			row = col.row(align=True)
			for visualizer in group.spr_visualizers:
				box = col.box()
				row = box.row(align=True)
				row.context_pointer_set("object", visualizer)
				row.prop(visualizer, "name",   text="")
				row.prop(visualizer, "type",   text="")
				row.prop(visualizer, "show",   text="", icon=('RESTRICT_VIEW_OFF' if visualizer.show==1 else 'RESTRICT_VIEW_ON'))
				row.operator("spr.cr_del_visualizer", text="", icon='X')

				c = box.column(align=True)
				if visualizer.type=="FOV":
					c.prop(visualizer, "target", text="Target")
					c.prop(visualizer, "fov_param_name_horiz", text="Horiz Param Name")
					c.prop(visualizer, "fov_param_name_vert",  text="Vert Param Name")

			col.operator("spr.cr_add_visualizer", text="Add Visualizer", icon='ZOOMIN')

# --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
# Groupパネルの上書き
# Override bl_ui/properties_object.py
class OBJECT_PT_groups(bpy.types.Panel):
	bl_space_type = 'PROPERTIES'
	bl_region_type = 'WINDOW'
	bl_context = "object"
	bl_label = "Groups"

	def draw(self, context):
		layout = self.layout

		ob = context.object

		row = layout.row(align=True)
		row.operator("object.group_link", text="Add to Group")
		row.operator("object.group_add", text="", icon='ZOOMIN')

		index = 0
		for group in bpy.data.groups:
			if ob.name in group.objects:
				col = layout.column(align=True)

				col.context_pointer_set("group", group)
				
				
				row = col.box().row()
				
				row.prop(group, "name", text="")
				row.operator("object.group_remove", text="", icon='X', emboss=False)

				split = col.box().split()

				col2 = split.column()
				col2.prop(group, "layers", text="Dupli")

				col2 = split.column()
				col2.prop(group, "dupli_offset", text="")

				props = col2.operator("object.dupli_offset_from_cursor", text="From Cursor")
				props.group = index
				index += 1
				
				#for Spr Creature
				split = col.box().split()
				cr_group = None
				if len(ob.users_group):
					for g in ob.users_group:
						if(g.spr_creature_group==1):
							cr_group = g
				if (cr_group is None) or (cr_group == group):
					split.prop(group,"spr_creature_group", text="Spr Creature")



## Creature Panelのボタン動作
class OBJECT_OT_SprCrAddParameter(bpy.types.Operator):
	bl_idname = "spr.cr_add_parameter"
	bl_label = "[spr]Add Creature Parameter"
	bl_description = "[Spr]"
	
	def execute(self,context):
		
		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		if(not group is None):
			param = group.spr_creature_parameters.add()
			param.creaturename = group.name
		
		return{'FINISHED'}

class OBJECT_OT_SprCrDelParameter(bpy.types.Operator):
	bl_idname = "spr.cr_del_parameter"
	bl_label = "[spr]Remove Creature Parameter"
	bl_description = "[Spr]"
	
	def execute(self,context):
		param = context.object
		parameters = bpy.data.groups[param.creaturename].spr_creature_parameters
		for i in range(0, len(parameters)):
			if parameters[i]==param:
				parameters.remove(i)
				return{'FINISHED'}
		
		return{'FINISHED'}


class OBJECT_OT_SprCrAddVisualizer(bpy.types.Operator):
	bl_idname = "spr.cr_add_visualizer"
	bl_label = "[spr]Add Creature Visualizer"
	bl_description = "[Spr]"
	
	def execute(self,context):
		
		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		if(not group is None):
			visualizer = group.spr_visualizers.add()
			visualizer.creaturename = group.name
		
		return{'FINISHED'}

class OBJECT_OT_SprCrDelVisualizer(bpy.types.Operator):
	bl_idname = "spr.cr_del_visualizer"
	bl_label = "[spr]Remove Creature Visualizer"
	bl_description = "[Spr]"
	
	def execute(self,context):
		visualizer  = context.object
		visualizers = bpy.data.groups[visualizer.creaturename].spr_visualizers
		for i in range(0, len(visualizers)):
			if visualizers[i]==visualizer:
				visualizers.remove(i)
				return{'FINISHED'}
		
		return{'FINISHED'}


class OBJECT_OT_SprCrVisualizeDisplay(bpy.types.Operator):
	bl_idname = "spr.cr_visualize_display"
	bl_label = "[spr]Creature Visualize Display"
	bl_description = "[Spr]"
	
	def execute(self,context):
		
		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		return{'FINISHED'}

class OBJECT_OT_SprCrVisualizeHide(bpy.types.Operator):
	bl_idname = "spr.cr_visualize_hide"
	bl_label = "[spr]Creature Visualize Hide"
	bl_description = "[Spr]"
	
	def execute(self,context):
		
		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		return{'FINISHED'}

class OBJECT_OT_SprCrRulesApply(bpy.types.Operator):
	bl_idname = "spr.cr_rules_apply"
	bl_label = "[spr]Creature Apply Rules"
	bl_description = "[Spr]"
	
	def execute(self,context):
		
		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		if(not group is None):
			creature_handler = spb.handlers.creature_handlers[group.name]
			if creature_handler:
				creature_handler.rule.update()
		
		return{'FINISHED'}

class OBJECT_OT_SprCrParametersApply(bpy.types.Operator):
	bl_idname = "spr.cr_parameters_apply"
	bl_label = "[spr]Creature Apply Parameters"
	bl_description = "[Spr]"
	
	def execute(self,context):
		
		obj = context.object
		group = None
		if len(obj.users_group):
			for g in obj.users_group:
				if(g.spr_creature_group==1):
					group = g
		
		if(not group is None):
			creature_handler = spb.handlers.creature_handlers[group.name]
			if creature_handler:
				creature_handler.apply_bpy_change = True
				creature_handler.sync()
		
		return{'FINISHED'}

