﻿#include "SprBlenderDLL.h"
#include "Physics/PHHapticEngineMultiBase.h"

// --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---

// シングルトン
SprBlenderDLL* SprBlenderDLL::instance = NULL;

// フックする前の本来のWndProc
WNDPROC WndProcOrig = NULL;

// --- --- --- --- ---

// シングルトン取得
SprBlenderDLL* SprBlenderDLL::GetInstance() {
	if (!instance) { instance = new SprBlenderDLL(); }
	return instance;
}

// タイマコールバック
void SprBlenderDLL::TimerCallback(int id, void* arg) {
	GetInstance()->TimerFunc(id);
}

void SprBlenderDLL::ThreadCallback(int id, void* arg) {
	GetInstance()->ThreadFunc(id);
}

// WndProcフック
LRESULT CALLBACK WndProcHook(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
	MSG m;
	m.hwnd		= hWnd;
	m.message	= msg;
	m.wParam	= wParam;
	m.lParam	= lParam;
	SprBlenderDLL* spb = SprBlenderDLL::GetInstance();

	// SpaceNavigatorが作成されていればメッセージを渡す
	for (int i=0; i<spb->NInterfaces(); ++i) {
		HISpaceNavigatorIf* spcnav = spb->GetInterface(i)->Cast();
		if (spcnav && spcnav->PreviewMessage(&m)) { return 0L; }
	}

	// SpaceNavigatorがメッセージを受け取らなかった場合は本来のWndProcに渡す
	return CallWindowProc(WndProcOrig, hWnd, msg, wParam, lParam);
}

// --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---

// コンストラクタ
SprBlenderDLL::SprBlenderDLL() {
	bInitialized = false;

	// --- --- --- --- ---
	// Create HISdk (何度も作らない)
	hiSdk = HISdkIf::CreateSdk();

	// --- --- --- --- ---
	// Add Real Devices
	DRUsb20SimpleDesc usbSimpleDesc;
	hiSdk->AddRealDevice(DRUsb20SimpleIf::GetIfInfoStatic(), &usbSimpleDesc);

	DRUsb20Sh4Desc usb20Sh4Desc;
	for (int i=0; i<10; ++i) {
		usb20Sh4Desc.channel = i;
		hiSdk->AddRealDevice(DRUsb20Sh4If::GetIfInfoStatic(), &usb20Sh4Desc);
	}

	DRCyUsb20Sh4Desc cyDesc;
	for(int i=0; i<10; ++i){
		cyDesc.channel = i;
		hiSdk->AddRealDevice(DRCyUsb20Sh4If::GetIfInfoStatic(), &cyDesc);
	}

	hiSdk->AddRealDevice(DRKeyMouseWin32If::GetIfInfoStatic());

	// --- --- --- --- ---
	// Hook WndProc
	hWnd = GetActiveWindow();
	WndProcOrig = (WNDPROC)(GetWindowLongPtr(hWnd, GWLP_WNDPROC));
	SetWindowLongPtr(hWnd, GWLP_WNDPROC, (LONG_PTR)(WndProcHook));	

	// --- --- --- --- ---
	// Create Timers
	
	// 物理ステップ用のタイマを作成
	timerPhysics = UTTimerIf::Create();
	timerPhysics->SetMode(UTTimerIf::THREAD);
	timerPhysics->SetCallback(SprBlenderDLL::ThreadCallback, this);
	timerPhysics->SetResolution(1);
	timerPhysics->SetInterval(1);
	timerPhysics->Stop();
	timerPhysicsID = timerPhysics->GetID();

	// 力覚スレッド用のマルチメディアタイマを作成
	timerHaptic = UTTimerIf::Create();
	timerHaptic->SetMode(UTTimerIf::MULTIMEDIA);
	timerHaptic->SetCallback(SprBlenderDLL::TimerCallback, this);
	timerHaptic->SetResolution(1);
	timerHaptic->SetInterval(1);
	timerHaptic->Stop();
	timerHapticID = timerHaptic->GetID();

	// --- --- --- --- ---
	// Prepare States
	states = ObjectStatesIf::Create();
}

// 初期化
void SprBlenderDLL::Init() {
	bInitialized = false;

	// Create SDKs (HISdkを除く)
	fwSdk = FWSdkIf::CreateSdk();
	crSdk = CRSdkIf::CreateSdk();

	// Create Scenes
	FWSceneIf* fwScene = fwSdk->CreateScene(PHSceneDesc(), GRSceneDesc());

	// Initialize PHScene
	PHSceneIf* phScene = fwScene->GetPHScene();
	phScene->SetHapticTimeStep((double)timerHaptic->GetInterval() / 1000.0);
	phScene->GetHapticEngine()->EnableHapticEngine(true);
	phScene->GetHapticEngine()->SetHapticEngineMode(PHHapticEngineDesc::MULTI_THREAD);

	bInitialized = true;
}

// --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---

// SPIDARの作成
HISpidarGIf* SprBlenderDLL::CreateSPIDAR() {
	HISpidarGIf* spidar;
	spidar = hiSdk->CreateHumanInterface(HISpidarGIf::GetIfInfoStatic())->Cast();
	spidar->Init(&HISpidarGDesc("SpidarG6X3R"));
	spidar->Calibration();

	interfaces.push_back(spidar);

	return spidar;
}

// SpaceNavigatorの作成
HISpaceNavigatorIf* SprBlenderDLL::CreateSpaceNavigator() {
	HISpaceNavigatorDesc descSpcNav;
	descSpcNav.hWnd = &hWnd;

	HISpaceNavigatorIf* spcNav;
	spcNav = hiSdk->CreateHumanInterface(HISpaceNavigatorIf::GetIfInfoStatic())->Cast();
	spcNav->Init(&descSpcNav);
	spcNav->SetPose(Posef(Vec3f(0,0,0), Quaternionf()));

	interfaces.push_back(spcNav);

	return spcNav;
}

// Novint Falconの作成
HINovintFalconIf* SprBlenderDLL::CreateNovintFalcon() {
	HINovintFalconIf* falcon;
	falcon = hiSdk->CreateHumanInterface(HINovintFalconIf::GetIfInfoStatic())->Cast();
	falcon->Init(NULL);
	falcon->Calibration();

	interfaces.push_back(falcon);

	return falcon;
}

FWHapticPointerIf* SprBlenderDLL::CreateHapticPointer(FWSceneIf* fwScene, HIBaseIf* hi) {
	// 力覚ポインタの作成
	PHHapticPointerIf* phPointer = fwScene->GetPHScene()->CreateHapticPointer();

	Posed defaultPose;
	defaultPose.Pos() = Vec3d(0,0,3);
	defaultPose.Ori() = Quaterniond::Rot(Rad(90), 'x');
	phPointer->SetDefaultPose(defaultPose);	// 力覚ポインタ初期姿勢の設定

	phPointer->SetLocalRange(0.1f);			// 局所シミュレーション範囲の設定
	phPointer->SetPosScale(100);			// 力覚ポインタの移動スケールの設定
	phPointer->SetReflexSpring(5000.0f);	// バネ係数の設定
	phPointer->SetReflexDamper(0.0f);		// ダンパ係数の設定
	phPointer->EnableFriction(false);		// 摩擦を有効にするかどうか
	phPointer->EnableForce(true);			// 力を有効にするかどうか

	// HumanInterfaceと接続するためのオブジェクトを作成
	FWHapticPointerIf* fwPointer = fwScene->CreateHapticPointer();

	fwPointer->SetHumanInterface(hi);		// HumanInterfaceの設定
	fwPointer->SetPHHapticPointer(phPointer); // PHHapticPointerIfの設定

	return fwPointer;
}

// --- --- --- --- ---

// タイマコールバック
void SprBlenderDLL::TimerFunc(int id) {
	if (!bInitialized) { return; }

	if (id==timerHapticID) {
		for (int i=0; i<fwSdk->NScene(); ++i) {
			fwSdk->GetScene(i)->UpdateHapticPointers();
			fwSdk->GetScene(i)->GetPHScene()->StepHapticLoop();
		}
	}
}

void SprBlenderDLL::ThreadFunc(int id) {
	if (!bInitialized) { return; }

	for (int i=0; i<fwSdk->NScene(); ++i) {
		PHHapticEngineIf* he = fwSdk->GetScene(i)->GetPHScene()->GetHapticEngine();

		he->StepPhysicsSimulation();
			
		if (he->IsAfterStepPhysicsSimulation()) {
			UTAutoLock lock(EPCriticalSection);

			crSdk->Step();

			{
				static int cycle = 0;
				static DWORD lastCounted = timeGetTime();
				DWORD now = timeGetTime();
				if (now - lastCounted > 1000) {
					cps = (float)(cycle) / (float)(now - lastCounted) * 1000.0f;
					lastCounted = now;
					cycle = 0;
				}
				cycle++;
			}
		}
	}
}

void SprBlenderDLL::StepPliantBefore() {
	PHSceneIf* phScene = GetFWSdk()->GetScene(0)->GetPHScene();
	jointdescs.resize(phScene->NJoints());

	states->SaveState(phScene);
	phScene->GetConstraintEngine()->EnableContactDetection(false);
	phScene->GetIKEngine()->Enable(true);

	for (size_t i=0; i<phScene->NJoints(); ++i) {
		PHHingeJointIf* hj = phScene->GetJoint(i)->Cast();
		if (hj) {
			jointdescs[i].hinge.spring = hj->GetSpring();
			jointdescs[i].hinge.damper = hj->GetDamper();
			hj->SetSpring(0);
			hj->SetDamper(FLT_MAX);
		}
		PHBallJointIf* bj = phScene->GetJoint(i)->Cast();
		if (bj) {
			jointdescs[i].ball.spring = bj->GetSpring();
			jointdescs[i].ball.damper = bj->GetDamper();
			bj->SetSpring(0);
			bj->SetDamper(FLT_MAX);
		}
	}

	phScene->Step();

	double limit = 100.0;
	for (size_t i=0; i<phScene->NJoints(); ++i) {
		PHHingeJointIf* hj = phScene->GetJoint(i)->Cast();
		if (hj) {
			hj->SetDamper(jointdescs[i].hinge.damper);
			jointdescs[i].hinge.offsetForce    = hj->GetMotorForce();
			if (jointdescs[i].hinge.offsetForce > limit) { jointdescs[i].hinge.offsetForce = limit; }
			jointdescs[i].hinge.targetPosition = hj->GetTargetPosition();			
			jointdescs[i].hinge.targetVelocity = hj->GetTargetVelocity();
		}
		PHBallJointIf* bj = phScene->GetJoint(i)->Cast();
		if (bj) {
			jointdescs[i].ball.offsetForce    = bj->GetMotorForce();
			if (jointdescs[i].ball.offsetForce.norm() > limit) { jointdescs[i].ball.offsetForce = jointdescs[i].ball.offsetForce.unit() * limit; }
			jointdescs[i].ball.targetPosition = bj->GetTargetPosition();			
			jointdescs[i].ball.targetVelocity = bj->GetTargetVelocity();
		}
	}

	states->LoadState(phScene);
	phScene->GetConstraintEngine()->EnableContactDetection(true);
	phScene->GetIKEngine()->Enable(false);

	for (size_t i=0; i<phScene->NJoints(); ++i) {
		PHHingeJointIf* hj = phScene->GetJoint(i)->Cast();
		if (hj) {
			hj->SetOffsetForce(jointdescs[i].hinge.offsetForce);
			hj->SetSpring(jointdescs[i].hinge.spring);
			hj->SetDamper(0);
			hj->SetTargetPosition(jointdescs[i].hinge.targetPosition);
			hj->SetTargetVelocity(jointdescs[i].hinge.targetVelocity);
		}
		PHBallJointIf* bj = phScene->GetJoint(i)->Cast();
		if (bj) {
			bj->SetOffsetForce(jointdescs[i].ball.offsetForce);
			bj->SetSpring(jointdescs[i].ball.spring);
			bj->SetDamper(0);
			bj->SetTargetPosition(jointdescs[i].ball.targetPosition);
			bj->SetTargetVelocity(jointdescs[i].ball.targetVelocity);
		}
	}
}

void SprBlenderDLL::StepPliantAfter() {
	PHSceneIf* phScene = GetFWSdk()->GetScene(0)->GetPHScene();

	for (size_t i=0; i<phScene->NJoints(); ++i) {
		PHHingeJointIf* hj = phScene->GetJoint(i)->Cast();
		if (hj) {
			hj->SetOffsetForce(0);
			hj->SetSpring(jointdescs[i].hinge.spring);
			hj->SetDamper(jointdescs[i].hinge.damper);
		}
		PHBallJointIf* bj = phScene->GetJoint(i)->Cast();
		if (bj) {
			bj->SetOffsetForce(Vec3d());
			bj->SetSpring(jointdescs[i].ball.spring);
			bj->SetDamper(jointdescs[i].ball.damper);
		}
	}
}