#include <Physics/PHHapticEngine.h>
#include <EmbPython/EmbPython.h>

#include <sstream>

#include "windows.h"

// --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
class SprBlenderDLL {

	// VOgIuWFNg
	static SprBlenderDLL* instance;

	// --- --- --- --- ---

	// SDK
	UTRef< FWSdkIf >        fwSdk;
	UTRef< CRSdkIf >        crSdk;
	UTRef< HISdkIf >        hiSdk;

	// Timer
	UTTimerIf*              timerHaptic;
	UTTimerIf*              timerPhysics;
	int                     timerHapticID;
	int                     timerPhysicsID;

	// CPS Counter
	float					cps;

	// C^tF[X
	std::vector< UTRef<HIBaseIf> >  interfaces;

	// EBhEnh
	HWND                    hWnd;

	// tO
	bool                    bInitialized;

	// For SaveState
	UTRef<ObjectStatesIf>	states;

	// For Pliant Motion
	struct JointDesc {
		PHHingeJointDesc hinge;
		PHBallJointDesc  ball;
	};
	std::vector<JointDesc>	jointdescs;

public:

	// VOg擾
	static SprBlenderDLL* GetInstance();

	// ^C}R[obN
	static void SPR_CDECL TimerCallback(int id, void* arg);
	static void SPR_CDECL ThreadCallback(int id, void* arg);
	
	// --- --- --- --- ---

	// RXgN^EfXgN^
	SprBlenderDLL();
	~SprBlenderDLL() {
		timerPhysics->Stop();
		timerHaptic->Stop();
	}

	// 
	void Init();

	// --- --- --- --- ---

	// SDK擾
	FWSdkIf*  GetFWSdk() { return fwSdk; }
	CRSdkIf*  GetCRSdk() { return crSdk; }
	HISdkIf*  GetHISdk() { return hiSdk; }

	// --- --- --- --- ---

	// C^tF[X擾
	int NInterfaces() { return (int)interfaces.size(); }
	HIBaseIf* GetInterface(int i) { return interfaces[i]; }

	// SPIDAR̍쐬
	HISpidarGIf* CreateSPIDAR();

	// SpaceNavigator̍쐬
	HISpaceNavigatorIf* CreateSpaceNavigator();

	// SpaceNavigator̍쐬
	HINovintFalconIf* CreateNovintFalcon();

	// HapticPointer̍쐬
	FWHapticPointerIf*  CreateHapticPointer(FWSceneIf* fwScene, HIBaseIf* hi);

	// ^C}擾
	UTTimerIf*  GetHapticTimer()  { return timerHaptic;  }
	UTTimerIf*  GetPhysicsTimer() { return timerPhysics; }

	// ^C}R[obN
	void TimerFunc(int id);
	void ThreadFunc(int id);

	// Step for Pliant Motion
	void StepPliantBefore();
	void StepPliantAfter();

	// <!!>ifobOpj ݂̃V[o
	void PrintScene() { fwSdk->Print(std::cout); }

	// CPS Count
	float GetCPS() { return cps; }

};
